/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : USB_BULK_Performance.c
Purpose : USB BULK sample showing a simply 1-byte echo server.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include <stdio.h>
#include "SEGGER.h"
#include "USB_Private.h"
#include "BSP.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define MAX_PACKET_SIZE      USB__IsHighSpeedMode() ? (512) : (64)

#define MAX_BYTES            (5 * MAX_PACKET_SIZE)
#define NUM_BYTES_BUFFER     0x2000

/*********************************************************************
*
*       Defines, non-configurable
*
**********************************************************************
*/
#define CMD_TEST_SPEED         0x01
#define CMD_TEST               0x02
#define SUBCMD_SPEED_READ       100
#define SUBCMD_SPEED_WRITE      101

#define MAX_NUM_BYTES_TEST_NET    (4100)

#define USB_BULK_ENDWRITE()

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static U32 _aMemBlock[NUM_BYTES_BUFFER / 4];

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _AddBULK
*
*  Function description
*    Add generic USB BULK interface to USB stack
*/
static void _AddBULK(void) {
  static U8 _abOutBuffer[USB_MAX_PACKET_SIZE];
  USB_BULK_INIT_DATA    InitData;

  InitData.EPIn  = USB_AddEP(1, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, NULL, 0);
  InitData.EPOut = USB_AddEP(0, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, _abOutBuffer, USB_MAX_PACKET_SIZE);
  USB_BULK_Add(&InitData);
}


/*********************************************************************
*
*       _ReadDummy
*
*  Function description
*    Reads data block via USB or ethernet, but simply ignores data
*
*  Return value
*    0     OK
*   -1     Error
*/
static int _ReadDummy(unsigned NumBytes) {
  U32 NumBytesAtOnce;
  int r;
  U8 abData[32];      // Dummy buffer

  do {
    NumBytesAtOnce = SEGGER_MIN(sizeof(abData), NumBytes);
    r = USB_BULK_Read(abData, NumBytesAtOnce);
    if (r) {
      return r;
    }
    NumBytes -= NumBytesAtOnce;
  } while (NumBytes);
  return 0;
}

/*********************************************************************
*
*       _WriteDummy
*
*  Function description
*    This function writes 0 data. Used in some cases to make sure we send the number of bytes expected to stay in-sync
*/
static void _WriteDummy(unsigned NumBytes) {
  U32 NumBytesAtOnce;
  U8  abData[32] = {0};

  do {
    NumBytesAtOnce = SEGGER_MIN(sizeof(abData), NumBytes);
    USB_BULK_Write(abData, NumBytesAtOnce);
    NumBytes -= NumBytesAtOnce;
  } while (NumBytes);
}

/*********************************************************************
*
*       _Delay
*
*/
static unsigned _Delay(volatile unsigned NumCycles) {
  do {} while (--NumCycles);
  return NumCycles;
}

/*********************************************************************
*
*       _Alloc
*
*/
static void * _Alloc(unsigned NumBytes) {
  if (NumBytes <= NUM_BYTES_BUFFER) {
    return _aMemBlock;
  }
  return NULL;
}

/*********************************************************************
*
*       _GetMaxBlock
*
*/
static unsigned _GetMaxBlock(void) {
  return sizeof(_aMemBlock);
}


/*********************************************************************
*
*       _Free
*
*/
static void _Free(void * p) {
  (void)p;
}


/*********************************************************************
*
*       _PerformanceTest
*
*  Function description:
*
*
*/
static void _TestNet(void) {
  U8  acIn[4];
  U8  SubCmd;
  int NumBytes;
  int NumBytesAtOnce;
  int Off;
  int OffNew;
  int NumBytesToSend;
  U8  r = 0;
  U8* pData;
  unsigned DelayusPerByte;
  //
  // Read the number of bytes
  //
  USB_BULK_Read(acIn, 3);
  SubCmd   = acIn[0];
  NumBytes = acIn[1] + (acIn[2] << 8);
  DelayusPerByte = (1 << (SubCmd & 15)) >> 1;
  //
  // Reserve space for data + one byte for status
  //
  pData = (U8*)_Alloc(NumBytes + 1);
  if (pData == NULL) {
    _ReadDummy(NumBytes);
    _WriteDummy(NumBytes + 1);
    NumBytesAtOnce = NumBytes;
    return;
  }
  //
  // Perform operation depending on transfer type
  //
  if ((SubCmd >> 4) == 0) {
    //
    // Read data overlapped
    //
    USB_BULK_ReadOverlapped(pData, NumBytes);
    Off = 0;
    do {
      //
      // Wait until a sufficient number of bytes has been received
      //
      do {
        int NumBytesRem = USB_BULK_GetNumBytesRemToRead();
        OffNew = NumBytes - NumBytesRem;
        NumBytesAtOnce = OffNew - Off;
        if (NumBytesRem == 0) {
          break;
        }
        if (NumBytesAtOnce >= MAX_BYTES) {
          NumBytesAtOnce &= ~(MAX_PACKET_SIZE  - 1);
          break;
        }
      } while (1);
      //
      // Delay the transfer to simulate JTAG transaction
      //
      if (DelayusPerByte) {
        USB_OS_Delay((NumBytesAtOnce * DelayusPerByte) >> 10);
      }
      r = 0;
      //
      // Send data back via USB (use same data as received)
      //
      NumBytesToSend = NumBytesAtOnce;
      //
      // Send status byte on last transfer
      //
      if (Off + NumBytesAtOnce == NumBytes) {   // Last transfer ?
        *(pData + Off + NumBytesAtOnce) = r;
        NumBytesToSend++;
      }
      USB_BULK_WriteOverlapped(pData + Off, NumBytesToSend);
      Off += NumBytesAtOnce;
    } while (Off != NumBytes);
    //
    // End USB transaction
    //
    USB_BULK_ENDWRITE();
  } else {
    //
    // Read data non-overlapped
    //
    USB_BULK_Read(pData, NumBytes);
    //
    // Delay the transfer to simulate work
    //
    if (DelayusPerByte) {
      _Delay(NumBytes * DelayusPerByte);
    }
    r = 0;
    //
    // Send data back via USB (use same data as received)
    //
    *(pData + NumBytes) = r;
    USB_BULK_Write(pData, NumBytes + 1);
  }
  //
  // Free the reserved memory
  //
  _Free(pData);
}

/*********************************************************************
*
*       _TestSpeed
*/
static void _TestSpeed(void) {
  U8  SubCmd;
  U32 NumReps;
  U32 NumBytes;
  U32 NumBytesRem;
  U32 NumBytesAtOnce;
  U32 AllocSize;
  int r;
  U8* p;
  U8  aData[8];

  USB_BULK_Read(&SubCmd,   1);
  USB_BULK_Read(aData, 8);
  //
  //  Get the command parameters
  //
  NumReps  = USB__GetU32LE(&aData[0]);
  NumBytes = USB__GetU32LE(&aData[4]);
  //
  // Check for valid parameters
  //
  if (NumBytes == 0) {
    return;
  }
  if ((NumReps | NumBytes) == 0) {
    return;
  }
  //
  //  Allocate a buffer in order to store or retrieve the data
  //
  AllocSize = NumBytes;
  if (AllocSize > _GetMaxBlock()) {
    AllocSize = _GetMaxBlock();
  }
  p = (U8*)_Alloc(AllocSize);
  if (p == NULL) {
    return;
  }
  //
  // Repeat the speed test acc. to NumReps.
  //
  do {
    NumBytesRem = NumBytes;
    //
    // Check which direction and execute command accordingly
    switch (SubCmd) {
    case SUBCMD_SPEED_WRITE:
      do {
        NumBytesAtOnce = SEGGER_MIN(NumBytesRem, AllocSize);
        NumBytesRem -= NumBytesAtOnce;
        r = USB_BULK_Read(p, NumBytesAtOnce);
        if (r < 0) {
          return;
        }
      } while (NumBytesRem);
      break;
    case SUBCMD_SPEED_READ:
      do {
        NumBytesAtOnce = SEGGER_MIN(NumBytesRem, AllocSize);
        NumBytesRem -= NumBytesAtOnce;
        USB_BULK_Write(p, NumBytesAtOnce);
      } while (NumBytesRem);
      break;
    default:
      return;
    }
  } while (--NumReps);
  //
  // Free the buffer
  //
  _Free(p);
}


/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       Get information that are used during enumeration
*/

/*********************************************************************
*
*       USB_GetVendorId
*
*  Function description
*    Returns vendor Id
*/
U16 USB_GetVendorId(void) {
  return 0x8765;
}

/*********************************************************************
*
*       USB_GetProductId
*
*  Function description
*    Returns product Id
*/
U16 USB_GetProductId(void) {
  return 0x1234;
}

/*********************************************************************
*
*       USB_GetVendorName
*
*  Function description
*    Returns vendor name
*/
const char * USB_GetVendorName(void) {
  return "Vendor";
}

/*********************************************************************
*
*       USB_GetProductName
*
*  Function description
*    Returns product name
*/
const char * USB_GetProductName(void) {
  return "Bulk device";
}

/*********************************************************************
*
*       USB_GetSerialNumber
*
*  Function description
*    Returns serial number
*/
const char * USB_GetSerialNumber(void) {
  return "123456790";
}


/*********************************************************************
*
*       MainTask
*
* Function description
*   USB handling task.
*   Modify to implement the desired protocol
*/
void MainTask(void);
void MainTask(void) {
  U8 Cmd;
  USB_Init();
  _AddBULK();
  USB_Start();
  //
  // Loop: Receive data byte by byte, send back (data + 1)
  //
  while (1) {
    //
    // Wait for configuration
    //
    while ((USB_GetState() & (USB_STAT_CONFIGURED | USB_STAT_SUSPENDED)) != USB_STAT_CONFIGURED) {
      BSP_ToggleLED(0);
      USB_OS_Delay(50);
    }
    BSP_SetLED(0);         // LED on to indicate we are waiting for data
    USB_BULK_Read(&Cmd, 1);
    switch (Cmd) {
    case CMD_TEST_SPEED:
      _TestSpeed();
      break;
    case CMD_TEST:
      _TestNet();
      break;
    }
  }
}
