/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : USB_MSD_FS_MT_Start.c
Purpose : Sample startup, storage driver for MSD using multitasking.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include "USB.h"
#include "USB_MSD.h"
#include "FS_Int.h"
#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
#define NUM_VOLUMES     2           // Number of volumes to be used as MSD
#define NUM_BUFFERS     2           // Number of transfer buffers, it has to be at least 2,
                                    // otherwise the behavior is undefined.
#define BUFFER_SIZE     (NUM_BUFFERS * ((8 * 1024) + USB_MSD_MT_WRITE_INFO_SIZE)) // Size of a transfer buffer in bytes

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static OS_STACKPTR int _aStackStorageTask[1024/sizeof(int)];
static OS_TASK         _TCBStorageTask;
static U32             _aBuffer[NUM_VOLUMES][BUFFER_SIZE / 4];
static OS_EVENT        _Event;

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/


/*********************************************************************
*
*       _FSTest
*/
static void _FSTest(void) {
  FS_FILE    * pFile;
  unsigned     Len;
  const char * sInfo = "This sample based on the Segger emUSB software with MSD component.\r\nFor further information please visit: www.segger.com\r\n";
  unsigned     NumVolumes;
  unsigned     iVolume;
  char         acVolumeName[20];

  Len        = strlen(sInfo);
  NumVolumes = FS_GetNumVolumes();
  for (iVolume = 0; iVolume < NumVolumes; ++iVolume) {
    FS_GetVolumeName(iVolume, &acVolumeName[0], sizeof(acVolumeName));
    if (FS_IsLLFormatted(acVolumeName) == 0) {
      FS_X_Log("Low level formatting");
      FS_FormatLow(acVolumeName);
    }
    if (FS_IsHLFormatted(acVolumeName) == 0) {
      FS_X_Log("High level formatting\n");
      FS_Format(acVolumeName, NULL);
    }
    strcat(acVolumeName, "\\Readme.txt");
    pFile = FS_FOpen(acVolumeName, "w");
    FS_Write(pFile, sInfo, Len);
    FS_FClose(pFile);
    FS_SetVolumeLabel(acVolumeName, "SEGGER");
    FS_Unmount(acVolumeName);
  }
}

/*********************************************************************
*
*       _AddMSD
*
*   Function description
*     Add one ore more mass storage devices to USB stack.
*/
static void _AddMSD(void) {
  static U8         _abOutBuffer[2 * USB_MAX_PACKET_SIZE];
  USB_MSD_INIT_DATA InitData;
  USB_MSD_INST_DATA InstData;
  int               NumVolumes;
  int               iVolume;
  char              acVolumeName[20];

  //
  // Allocated 1 in and 1 out end point for the MSD module.
  //
  InitData.EPIn  = USB_AddEP(1, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, NULL, 0);
  InitData.EPOut = USB_AddEP(0, USB_TRANSFER_TYPE_BULK, USB_MAX_PACKET_SIZE, _abOutBuffer, sizeof(_abOutBuffer));
  USB_MSD_Add(&InitData);
  //
  // For each file system module create a MSD instance.
  //
  NumVolumes = FS_GetNumVolumes();
  NumVolumes = MIN(NumVolumes, NUM_VOLUMES);
  for (iVolume = 0; iVolume < NumVolumes; ++iVolume) {
    //
    // Create the MSD unit.
    //
    FS_GetVolumeName(iVolume, &acVolumeName[0], sizeof(acVolumeName));
    memset(&InstData, 0,  sizeof(InstData));
    InstData.pAPI              = &USB_MSD_StorageMT;
    InstData.DriverData.pStart = (void *)acVolumeName;
    InstData.DriverData.StartSector     = 0;
    InstData.DriverData.NumBytes4Buffer = BUFFER_SIZE;
    InstData.DriverData.NumBuffers      = NUM_BUFFERS;
    InstData.DriverData.pSectorBuffer   = _aBuffer[iVolume];
    USB_MSD_AddUnit(&InstData);
  }
}

/*********************************************************************
*
*       _SetEvent
*
*  Function description:
*    This function is used by the MSD MT layer in order to wake up the
*    MT_Storage task for handling a read or write operation.
*
*/
static void _SetEvent(void) {
  OS_EVENT_Set(&_Event);
}

/*********************************************************************
*
*       _WaitEvent
*
*  Function description:
*    This function is used by the USB MSD Storagetask to wait for
*    read/write operation event.
*
*/
static void _WaitEvent(void) {
  OS_EVENT_Wait(&_Event);
}

/*********************************************************************
*
*       _EventApi
*
*/
static const USB_MSD_EVENT_API _EventApi = {
  _WaitEvent,
  _SetEvent
};


/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       Get information that are used during enumeration
*/

/*********************************************************************
*
*       USB_GetVendorName
*
*   Function description
*     Returns vendor Id
*/
U16 USB_GetVendorId(void) {
  return 0x8765;
}

/*********************************************************************
*
*       USB_GetProductId
*
*   Function description
*     Returns the product Id
*/
U16 USB_GetProductId(void) {
  return 0x1000;
}

/*********************************************************************
*
*       USB_GetVendorName
*
*   Function description
*     Returns vendor name.
*/
const char * USB_GetVendorName(void) {
  return "Vendor";
}

/*********************************************************************
*
*       USB_GetProductName
*
*   Function description
*     Returns product name
*/
const char * USB_GetProductName(void) {
  return "MSD device";
}

/*********************************************************************
*
*       USB_GetSerialNumber
*
*   Function description
*     Returns serial number
*/
const char * USB_GetSerialNumber(void) {
  return "000013245678";  // Should be 12 character or more for compliance with Mass Storage Device Bootability spec.
}

/*********************************************************************
*
*       String information routines when inquiring the volume
*/
/*********************************************************************
*
*       USB_MSD_GetVendorName
*/
const char * USB_MSD_GetVendorName(U8 Lun) {
  FS_USE_PARA(Lun);
  return "Vendor";
}

/*********************************************************************
*
*       USB_MSD_GetProductName
*/
const char * USB_MSD_GetProductName(U8 Lun) {
  FS_USE_PARA(Lun);
  return "MSD Volume";
}

/*********************************************************************
*
*       USB_MSD_GetProductVer
*/
const char * USB_MSD_GetProductVer(U8 Lun) {
  FS_USE_PARA(Lun);
  return "1.00";
}

/*********************************************************************
*
*       USB_MSD_GetSerialNo
*/
const char * USB_MSD_GetSerialNo(U8 Lun) {
  FS_USE_PARA(Lun);
  return "134657890";
}

/*********************************************************************
*
*       MainTask
*
*/
void MainTask(void);
void MainTask(void) {
  U8 Prio;

  USB_Init();
  FS_Init();
  _FSTest();
  _AddMSD();
  //
  // Create a task that handles the writes sectors of the MSD stack
  //
  Prio = OS_GET_PRIORITY(OS_Global.pTask) + 1;
  OS_EVENT_Create(&_Event);
  USB_MSD_Storage_MTInit(&_EventApi);
  OS_CREATETASK(&_TCBStorageTask, "StorageTask", USB_MSD_StorageTask, Prio, _aStackStorageTask);
  USB_Start();
  while (1) {
    //
    // Wait for configuration
    //
    while ((USB_GetState() & (USB_STAT_CONFIGURED | USB_STAT_SUSPENDED)) != USB_STAT_CONFIGURED) {
      BSP_ToggleLED(0);
      USB_OS_Delay(50);
    }
    BSP_SetLED(0);
    USB_MSD_Task();
  }
}

/**************************** end of file ***************************/

