/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : USB.h
Purpose : USB stack API
          Do not modify to allow easy updates !
--------  END-OF-HEADER  ---------------------------------------------
*/

#ifndef USB_H     /* Avoid multiple inclusion */
#define USB_H

#include "SEGGER.h"
#include "USB_Conf.h"

#if defined(__cplusplus)
extern "C" {     /* Make sure we have C-declarations in C++ programs */
#endif

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

/* USB system version */
#define USB_VERSION                24004UL

/*********************************************************************
*
*       Default values
*
*/
#ifndef   USB_SUPPORT_HIGH_SPEED
  #define USB_SUPPORT_HIGH_SPEED 1
#endif

#ifndef   USB_MAX_PACKET_SIZE
  #if USB_SUPPORT_HIGH_SPEED
    #define USB_MAX_PACKET_SIZE   512
  #else
    #define USB_MAX_PACKET_SIZE    64
  #endif
#endif

#ifndef   USB_DEBUG_LEVEL
  #define USB_DEBUG_LEVEL  0
#endif

#ifndef    USB_DRIVER_OPTIMIZE
  #define  USB_DRIVER_OPTIMIZE
#endif

#define USB_STATUS_ERROR       -1
#define USB_STATUS_TIMEOUT     -2
#define USB_STATUS_EP_HALTED   -3
#define USB_STATUS_TRIAL       -255

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/
#define USB_Driver_NEC_70F377x    USB_Driver_NEC_70F376x

/*********************************************************************
*
*       Transfer types
*/
#define USB_TRANSFER_TYPE_CONTROL   0     // Refer to 9.6.6, table 9-13
#define USB_TRANSFER_TYPE_ISO       1
#define USB_TRANSFER_TYPE_BULK      2
#define USB_TRANSFER_TYPE_INT       3

/*********************************************************************
*
*       Endpoint direction
*/
#define USB_DIR_IN  1
#define USB_DIR_OUT 0

/*********************************************************************
*
*       Status flags
*/
#define USB_STAT_ATTACHED   (1 << 4)
#define USB_STAT_READY      (1 << 3)      // Set by any bus reset. Is required to go from "Powered" to "Addressed"
#define USB_STAT_ADDRESSED  (1 << 2)
#define USB_STAT_CONFIGURED (1 << 1)
#define USB_STAT_SUSPENDED  (1 << 0)

/*********************************************************************
*
*       Driver commands
*/
#define USB_DRIVER_CMD_SET_CONFIGURATION        0
#define USB_DRIVER_CMD_GET_TX_BEHAVIOR          1
#define USB_DRIVER_CMD_GET_SETADDRESS_BEHAVIOR  2
#define USB_DRIVER_CMD_REMOTE_WAKEUP            3
#define USB_DRIVER_CMD_TESTMODE                 4

#define USB_CMD_TESTMODE_TEST_J                 1
#define USB_CMD_TESTMODE_TEST_K                 2
#define USB_CMD_TESTMODE_TEST_SE0_NAK           3
#define USB_CMD_TESTMODE_TEST_PACKET            4
#define USB_CMD_TESTMODE_TEST_FORCE_ENABLE      5

/*********************************************************************
*
*       Types / structures
*/
enum {
  STRING_INDEX_LANGUAGE = 0,  // Language index. MUST BE 0 acc. to spec.
  STRING_INDEX_MANUFACTURER,  // iManufacturer:      Index of String Desc (Manuf)    (variable, but needs to be unique)
  STRING_INDEX_PRODUCT,       // iProduct:           Index of String Desc (Product)  (variable, but needs to be unique)
  STRING_INDEX_SN,            // iSerialNumber:      Index of String Desc (Serial #) (variable, but needs to be unique)
  STRING_INDEX_CONFIG,        // iConfiguration:     Index of String Desc (Configuration name) (variable, but needs to be unique)
  STRING_INDEX_INTERFACE      // iInterface:         Index of String Desc (Interface name) (variable, but needs to be unique)
};


typedef struct _USB_INFO_BUFFER USB_INFO_BUFFER;
typedef struct _EP_STAT EP_STAT;

typedef struct {
  U8 bmRequestType;
  U8 bRequest;
  U8 wValueLow;
  U8 wValueHigh;
  U8 wIndexLow;
  U8 wIndexHigh;
  U8 wLengthLow;
  U8 wLengthHigh;
} USB_SETUP_PACKET;

typedef struct {
  U8 EPIn;
  U8 EPOut;
} USB_BULK_INIT_DATA;

typedef struct _USB_HOOK {
  struct _USB_HOOK * pNext;
  void    (*cb) (void* pContext, U8 NewState);
  void     * pContext;
} USB_HOOK;

typedef struct _USB_HW_DRIVER {
  void     (*pfInit)                (void);
  U8       (*pfAllocEP)             (U8  InDir, U8 TransferType);
  void     (*pfUpdateEP)            (EP_STAT * pEPStat);
  void     (*pfEnable)              (void);
  void     (*pfAttach)              (void);
  unsigned (*pfGetMaxPacketSize)    (U8  EPIndex);
  int      (*pfIsInHighSpeedMode)   (void);
  void     (*pfSetAddress)          (U8  Addr);
  void     (*pfSetClrStallEP)       (U8  EPIndex, int OnOnff);
  void     (*pfStallEP0)            (void);
  void     (*pfDisableRxInterruptEP)(U8  EPIndex);
  void     (*pfEnableRxInterruptEP) (U8  EPIndex);
  void     (*pfStartTx)             (U8  EPIndex);
  void     (*pfSendEP)              (U8  EPIndex, const U8 * p, unsigned NumBytes);
  void     (*pfDisableTx)           (U8  EPIndex);
  void     (*pfResetEP)             (U8  EPIndex);
  int      (*pfControl)             (int Cmd, void * p);
  int      (*pfDeInit)              (void);
  int      (*pfDetach)              (void);
} USB_HW_DRIVER;

typedef int          USB_ON_CLASS_REQUEST   (const USB_SETUP_PACKET * pSetupPacket);
typedef int          USB_ON_SETUP           (const USB_SETUP_PACKET * pSetupPacket);
typedef void         USB_ADD_FUNC_DESC      (int InterfaceNo, USB_INFO_BUFFER * pInfoBuffer);
typedef void         USB_ON_RX_FUNC         (const U8 * pData, unsigned NumBytes);
typedef void         USB_ISR_HANDLER        (void);
typedef void         USB_DETACH_FUNC        (void);
typedef const char * USB_GET_STRING_FUNC    (int Index);
typedef U16          USB_ON_BCD_VERSION_FUNC(void);

/*********************************************************************
*
*       Public API functions
*
*/
unsigned USB_AddEP                   (U8 InDir, U8 TransferType, U16 Interval, U8 * pBuffer, unsigned BufferSize);
void     USB_DeInit                  (void);
void     USB_EnableIAD               (void);
int      USB_GetState                (void);
void     USB_Init                    (void);
char     USB_IsConfigured            (void);
void     USB_SetMaxPower             (unsigned MaxPower);
void     USB_Start                   (void);
void     USB_Stop                    (void);
int      USB_RegisterSCHook          (USB_HOOK * pHook, void (*cb) (void* pContext, U8 NewState), void * pContext);
int      USB_UnregisterSCHook        (USB_HOOK * pHook);

void     USB_CancelIO                (U8 EPIndex);
unsigned USB_GetNumBytesInBuffer     (U8 EPIndex);
int      USB_ReadEP                  (U8 EPOut, void * pData, unsigned NumBytesReq);
int      USB_ReadEPOverlapped        (U8 EPOut, void * pData, unsigned NumBytesReq);
int      USB_ReadEPTimed             (U8 EPOut, void * pData, unsigned NumBytesReq, unsigned ms);
int      USB_ReceiveEP               (U8 EPOut, void * pData, unsigned NumBytesReq);
int      USB_ReceiveEPTimed          (U8 EPOut, void* pData, unsigned NumBytesReq, unsigned ms);
void     USB_SetOnRXHookEP           (U8 EPIndex, USB_ON_RX_FUNC * pfOnRx);
void     USB_SetClrStallEP           (U8 EPIndex, int OnOff);
void     USB_StallEP                 (U8 EPIndex);
void     USB_WaitForEndOfTransfer    (U8 EPIndex);
int      USB_WriteEP                 (U8 EPIndex, const void* pData, unsigned NumBytes, char Send0PacketIfRequired);
int      USB_WriteEPOverlapped       (U8 EPIndex, const void* pData, unsigned NumBytes, char Send0PacketIfRequired);
int      USB_WriteEPTimed            (U8 EPIndex, const void* pData, unsigned NumBytes, char Send0PacketIfRequired, unsigned ms);
void     USB_StartReadTransfer       (U8 EPIndex);



void     USB_SetAddFuncDesc          (USB_ADD_FUNC_DESC    * pfAddDescFunc);
void     USB_SetClassRequestHook     (unsigned InterfaceNum, USB_ON_CLASS_REQUEST * pfOnClassRequest);
void     USB_SetVendorRequestHook    (unsigned InterfaceNum, USB_ON_CLASS_REQUEST * pfOnVendorRequest);
void     USB_SetOnSetupHook          (unsigned InterfaceNum, USB_ON_SETUP         * pfOnSetup);
void     USB_SetOnRxEP0              (USB_ON_RX_FUNC       * pfOnRx);
void     USB_SetDetachFunc           (USB_DETACH_FUNC * pfDetach);
void     USB_SetGetStringFunc        (USB_GET_STRING_FUNC * pfOnGetString);
void     USB_SetOnBCDVersionFunc     (USB_ON_BCD_VERSION_FUNC * pfOnGetString);

void     USB_DoRemoteWakeup          (void);
void     USB_SetIsSelfPowered        (U8 IsSelfPowered);
void     USB_SetAllowRemoteWakeUp    (U8 AllowRemoteWakeup);

/*********************************************************************
*
*       Public BULK API functions
*
*/
void     USB_BULK_Init                (void);
void     USB_BULK_Add                 (const USB_BULK_INIT_DATA * pInitData);
void     USB_BULK_CancelRead          (void);
void     USB_BULK_CancelWrite         (void);
unsigned USB_BULK_GetNumBytesInBuffer (void);
unsigned USB_BULK_GetNumBytesRemToRead(void);
unsigned USB_BULK_GetNumBytesToWrite  (void);
int      USB_BULK_Read                (      void * pData, unsigned NumBytes);
int      USB_BULK_ReadOverlapped      (      void * pData, unsigned NumBytes);
int      USB_BULK_ReadTimed           (      void * pData, unsigned NumBytes, unsigned ms);
int      USB_BULK_Receive             (      void * pData, unsigned NumBytes);
int      USB_BULK_ReceiveTimed        (      void * pData, unsigned NumBytes, unsigned ms);
void     USB_BULK_SetOnRXHook         (USB_ON_RX_FUNC * pfOnRx);
void     USB_BULK_WaitForRX           (void);
void     USB_BULK_WaitForTX           (void);
int      USB_BULK_Write               (const void * pData, unsigned NumBytes);
int      USB_BULK_WriteEx             (const void * pData, unsigned NumBytes, char Send0PacketIfRequired);
int      USB_BULK_WriteExTimed        (const void * pData, unsigned NumBytes, char Send0PacketIfRequired, unsigned ms);
int      USB_BULK_WriteOverlapped     (const void * pData, unsigned NumBytes);
int      USB_BULK_WriteOverlappedEx   (const void * pData, unsigned NumBytes, char Send0PacketIfRequired);
int      USB_BULK_WriteTimed          (const void * pData, unsigned NumBytes, unsigned ms);
void     USB_BULK_WriteNULLPacket     (void);

/*********************************************************************
*
*       Kernel interface routines (also for polled mode without kernel
*
*/
void     USB_OS_Init                   (void);
void     USB_OS_Delay                  (int ms);
void     USB_OS_DecRI                  (void);
U32      USB_OS_GetTickCnt             (void);
void     USB_OS_IncDI                  (void);
void     USB_OS_Panic                  (unsigned ErrCode);
void     USB_OS_Signal                 (unsigned EPIndex);
void     USB_OS_Wait                   (unsigned EPIndex);
int      USB_OS_WaitTimed              (unsigned EPIndex, unsigned ms);

/*********************************************************************
*
*       Get String functions during enumeration
*
*/
const char * USB_GetVendorName  (void);
const char * USB_GetProductName (void);
const char * USB_GetSerialNumber(void);

/*********************************************************************
*
*       Get Vendor/Product Ids during enumeration
*
*/
U16  USB_GetVendorId(void);
U16  USB_GetProductId(void);


/*********************************************************************
*
*       Setting USB target driver
*
*/
void USB_AddDriver(const USB_HW_DRIVER * pDriver);


/*********************************************************************
*
*       Function that has to be supplied by the customer
*
*/
void USB_X_AddDriver(void);
void USB_X_HWAttach(void);
void USB_X_EnableISR(USB_ISR_HANDLER * pfISRHandler);

/*********************************************************************
*
*       Individual driver configuration functions
*
*/
void USB_DRIVER_LPC17xx_ConfigAddr(U32 BaseAddr);
void USB_DRIVER_LPC18xx_ConfigAddr(U32 BaseAddr);
void USB_DRIVER_LPC43xx_ConfigAddr(U32 BaseAddr);
void USB_DRIVER_RX62N_ConfigAddr  (U32 BaseAddr);

void USB_DRIVER_STM32F4xxHS_ConfigPHY(U8 UsePHY);

/*********************************************************************
*
*       Available target USB drivers
*
*/

#define USB_Driver_AtmelCAP9        USB_Driver_Atmel_CAP9
#define USB_Driver_AtmelSAM3U       USB_Driver_Atmel_SAM3U
#define USB_Driver_AtmelRM9200      USB_Driver_Atmel_RM9200
#define USB_Driver_AtmelSAM7A3      USB_Driver_Atmel_SAM7A3
#define USB_Driver_AtmelSAM7S       USB_Driver_Atmel_SAM7S
#define USB_Driver_AtmelSAM7SE      USB_Driver_Atmel_SAM7SE
#define USB_Driver_AtmelSAM7X       USB_Driver_Atmel_SAM7X
#define USB_Driver_AtmelSAM9260     USB_Driver_Atmel_SAM9260
#define USB_Driver_AtmelSAM9261     USB_Driver_Atmel_SAM9261
#define USB_Driver_AtmelSAM9263     USB_Driver_Atmel_SAM9263
#define USB_Driver_AtmelSAM9G45     USB_Driver_Atmel_SAM9G45
#define USB_Driver_AtmelSAM9G20     USB_Driver_Atmel_SAM9G20
#define USB_Driver_AtmelSAM9Rx64    USB_Driver_Atmel_SAM9Rx64
#define USB_Driver_AtmelSAM9XE      USB_Driver_Atmel_SAM9XE
#define USB_Driver_NXPLPC13xx       USB_Driver_NXP_LPC13xx
#define USB_Driver_NXPLPC17xx       USB_Driver_NXP_LPC17xx
#define USB_Driver_NXPLPC214x       USB_Driver_NXP_LPC214x
#define USB_Driver_NXPLPC23xx       USB_Driver_NXP_LPC23xx
#define USB_Driver_NXPLPC24xx       USB_Driver_NXP_LPC24xx
#define USB_Driver_NXPLPC288x       USB_Driver_NXP_LPC288x
#define USB_Driver_NXPLPC318x       USB_Driver_NXP_LPC318x
#define USB_Driver_NXPLPC313x       USB_Driver_NXP_LPC313x
#define USB_Driver_OKI69Q62         USB_Driver_OKI_69Q62
#define USB_Driver_SharpLH79524     USB_Driver_Sharp_LH79524
#define USB_Driver_SharpLH7A40x     USB_Driver_Sharp_LH7A40x
#define USB_Driver_STSTM32          USB_Driver_ST_STM32
#define USB_Driver_STSTM32F107      USB_Driver_ST_STM32F107
#define USB_Driver_STSTR71x         USB_Driver_ST_STR71x
#define USB_Driver_STSTR750         USB_Driver_ST_STR750
#define USB_Driver_STSTR91x         USB_Driver_ST_STR91x
#define USB_Driver_H8SX1668R        USB_Driver_Renesas_H8SX1668R
#define USB_Driver_H8S2472          USB_Driver_Renesas_H8S2472
#define USB_Driver_TMPA910          USB_Driver_Toshiba_TMPA910
#define USB_Driver_TMPA900          USB_Driver_Toshiba_TMPA900
#define USB_Driver_RX62N            USB_Driver_Renesas_RX62N
#define USB_Driver_SH7203           USB_Driver_Renesas_SH7203
#define USB_Driver_SH7216           USB_Driver_Renesas_SH7216
#define USB_Driver_SH7286           USB_Driver_Renesas_SH7286
#define USB_Driver_Renesas_RX63N    USB_Driver_Renesas_RX62N
#define USB_Driver_STSTM32F2xxFS    USB_Driver_ST_STM32F2xxFS
#define USB_Driver_STSTM32F2xxHS    USB_Driver_ST_STM32F2xxHS
#define USB_Driver_Fujitsu_MB9BF506 USB_Driver_Fujitsu_MB9BFxxx
#define USB_Driver_FujitsuMB9BF50x  USB_Driver_Fujitsu_MB9BFxxx
#define USB_Driver_Renesas_SH7269   USB_Driver_Renesas_SH7268

extern const USB_HW_DRIVER USB_Driver_Dummy;
extern const USB_HW_DRIVER USB_Driver_Atmel_AT32UC3x;
extern const USB_HW_DRIVER USB_Driver_Atmel_CAP9;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM3U;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM3X;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM3S;
extern const USB_HW_DRIVER USB_Driver_Atmel_RM9200;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM7A3;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM7S;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM7SE;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM7X;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9260;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9261;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9263;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9G45;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9G20;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9Rx64;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAM9XE;
extern const USB_HW_DRIVER USB_Driver_EM_EFM32GG990;
extern const USB_HW_DRIVER USB_Driver_Freescale_iMX25x;
extern const USB_HW_DRIVER USB_Driver_Freescale_K40;
extern const USB_HW_DRIVER USB_Driver_Freescale_K60;
extern const USB_HW_DRIVER USB_Driver_Freescale_K70;
extern const USB_HW_DRIVER USB_Driver_Freescale_iMX28x;
extern const USB_HW_DRIVER USB_Driver_Freescale_MCF227x;
extern const USB_HW_DRIVER USB_Driver_Freescale_MCF225x;
extern const USB_HW_DRIVER USB_Driver_Freescale_MCF51JMx;
extern const USB_HW_DRIVER USB_Driver_Freescale_Vybrid;
extern const USB_HW_DRIVER USB_Driver_Fujitsu_MB9BFxxx;
extern const USB_HW_DRIVER USB_Driver_NEC_70F376x;
extern const USB_HW_DRIVER USB_Driver_NEC_uPD720150;
extern const USB_HW_DRIVER USB_Driver_NEC_78F102x;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC13xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC17xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC214x;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC23xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC24xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC288x;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC318x;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC313x;
extern const USB_HW_DRIVER USB_Driver_OKI_69Q62;
extern const USB_HW_DRIVER USB_Driver_Sharp_LH79524;
extern const USB_HW_DRIVER USB_Driver_Sharp_LH7A40x;
extern const USB_HW_DRIVER USB_Driver_ST_STM32;
extern const USB_HW_DRIVER USB_Driver_ST_STM32F107;
extern const USB_HW_DRIVER USB_Driver_ST_STM32F4xxFS;
extern const USB_HW_DRIVER USB_Driver_ST_STM32F4xxHS;
extern const USB_HW_DRIVER USB_Driver_ST_STM32F2xxFS;
extern const USB_HW_DRIVER USB_Driver_ST_STM32F2xxHS;
extern const USB_HW_DRIVER USB_Driver_ST_STR71x;
extern const USB_HW_DRIVER USB_Driver_ST_STR750;
extern const USB_HW_DRIVER USB_Driver_ST_STR91x;
extern const USB_HW_DRIVER USB_Driver_Renesas_H8SX1668R;
extern const USB_HW_DRIVER USB_Driver_Renesas_H8S2472;
extern const USB_HW_DRIVER USB_Driver_Renesas_RX62N;
extern const USB_HW_DRIVER USB_Driver_Renesas_SH7203;
extern const USB_HW_DRIVER USB_Driver_Renesas_SH7216;
extern const USB_HW_DRIVER USB_Driver_Renesas_SH7286;
extern const USB_HW_DRIVER USB_Driver_TI_MSP430;
extern const USB_HW_DRIVER USB_Driver_TI_LM3S9B9x;
extern const USB_HW_DRIVER USB_Driver_Toshiba_TMPM369;
extern const USB_HW_DRIVER USB_Driver_Toshiba_TMPA900;
extern const USB_HW_DRIVER USB_Driver_Toshiba_TMPA910;
extern const USB_HW_DRIVER USB_Driver_Wizcom;
extern const USB_HW_DRIVER USB_Driver_Renesas_uPD70F351x;
extern const USB_HW_DRIVER USB_Driver_Renesas_SH7268;
extern const USB_HW_DRIVER USB_Driver_Microchip_PIC32;
extern const USB_HW_DRIVER USB_Driver_Freescale_KinetisEHCI;
extern const USB_HW_DRIVER USB_Driver_Infineon_XMC45xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC18xx;
extern const USB_HW_DRIVER USB_Driver_NXP_LPC43xx;
extern const USB_HW_DRIVER USB_Driver_Renesas_RX100;
extern const USB_HW_DRIVER USB_Driver_Renesas_RL78;
extern const USB_HW_DRIVER USB_Driver_TI_OMAP_L138;
extern const USB_HW_DRIVER USB_Driver_TI_AM335x;
extern const USB_HW_DRIVER USB_Driver_Atmel_SAMA5Dx;
extern const USB_HW_DRIVER USB_Driver_TI_TM4Cxx;
#if defined(__cplusplus)
  }              /* Make sure we have C-declarations in C++ programs */
#endif

#endif
